//
//  STELSOCK	(SSL Telnet Socket)
//
//	Copyright (C) 1996,1997 NEC Corporation
//	All rights reserved.
//  Authos(s): Mizukoshi, Yasuhiro <mizukosi@ccs.mt.nec.co.jp>
//

static char CopyRightID[] = "Copyright(C) 1996,1997 NEC Corporation. All rights reserved.";

//	    $Archive: /sotools/stelsock/stelsock.c $
//			$Revision: 5 $
//			$Date: 97/05/15 1:02p $
static char VssID[] = "$Header: /sotools/stelsock/stelsock.c 5     97/05/15 1:02p Mizukosi $";

//
//	Comment:
//		This DLL makes telnet application possible to communicate
//		SSL telnetd.
//

//
//	Functions:
//		closesocket
//		connect
//		recv
//		send
//		WSAAsyncSelect
//		InitLocalData
//		DllMain
//		LoadWinsock
//		FreeWinsock
//

#include	<windows.h>
#include	<winsock.h>
#include	<stdio.h>

#include	"ssl.h"
#include	"stelsock.h"
#include	"auth.h"

#define		GLOBALS
#include	"globals.h"


/////////////////////////////////////////////////////////////////
//
//	closesocket
//		Free the context of SSL telnet
//
int PASCAL FAR EXPORT
closesocket (SOCKET s)
{
	int		ret;

x_OutputDebugString("closesocket ");

	ret = gft.closesocket(s);

	SocketListDelete(s);

	return ret;
}


///////////////////////////////////////////////////////////////////
//
//	connect
//
int PASCAL FAR EXPORT
connect (SOCKET s, const struct sockaddr FAR *name, int namelen)
{
	int		ret;
	unsigned long nbmode;
	struct LocalData		*ld;
	struct SocketList		*psl;
	UINT	uAsyncMsg;
	long	lAsyncEvent;

	ld  = (struct LocalData *)TlsGetValue(TlsIndex);
	psl = SocketListGetPos(s);

x_OutputDebugString("connect ");

	// Cancel async message temporally
	if (ld->hAsyncWnd){
		// Save async message and event.
		uAsyncMsg	= ld->uAsyncMsg;
		lAsyncEvent	= ld->lAsyncEvent;

		WSAAsyncSelect(ld->sAsyncSocket, ld->hAsyncWnd, 0, 0);
	}

	// change to blocking-mode
	nbmode = FALSE;
	ioctlsocket(s, FIONBIO, &nbmode);

	// Connect to telnet server
	ret = gft.connect(s, name, namelen);

	// SSL telnet negotiation
	if (!ret && psl && DoSSLTelnet(s)){

		psl->STelChk.ctx = SSL_CTX_new();
		psl->STelChk.con = (SSL *)SSL_new(psl->STelChk.ctx);

		SSL_set_fd(psl->STelChk.con, s);

		if (SSL_connect(psl->STelChk.con) <= 0){
x_OutputDebugString("\n# SSL Not Active. #####\n");

		} else {
			BOOL	bOpt;

x_OutputDebugString("\n# SSL Active. #####\n");

			psl->STelChk.bSSLActive = TRUE;

			bOpt = TRUE;
			if (setsockopt(s, SOL_SOCKET, SO_OOBINLINE, (char *)&bOpt, sizeof(bOpt))
								== SOCKET_ERROR){
x_OutputDebugString("setsockopt error! ");
			}

		}

	}

	if (!ret && ld->hAsyncWnd && (lAsyncEvent&FD_CONNECT) == FD_CONNECT){
		PostMessage(ld->hAsyncWnd, uAsyncMsg, ld->sAsyncSocket, FD_CONNECT);
	}

	return ret;
}


/////////////////////////////////////////////////////////////////
//
//	recv
//
int PASCAL FAR EXPORT
recv (SOCKET s, char FAR * buf, int len, int flags)
{
	int		ret;
	struct LocalData		*ld;
	struct SocketList		*psl;

	ld  = (struct LocalData *)TlsGetValue(TlsIndex);
	psl = SocketListGetPos(s);

x_OutputDebugString("recv ");

	if (psl && psl->STelChk.bSSLActive && !psl->STelChk.bInSSLCall){
x_OutputDebugString("S ");

		psl->STelChk.bInSSLCall = TRUE;
		ret = SSL_read(psl->STelChk.con, buf, len);
		psl->STelChk.bInSSLCall = FALSE;

	} else {
x_OutputDebugString("O ");

		ret = gft.recv(s, buf, len, flags);

	}

	return ret;
}


/////////////////////////////////////////////////////////////////
//
//	send
//
int PASCAL FAR EXPORT
send (SOCKET s, const char FAR * buf, int len,
    int flags)
{
	int		ret;
	struct LocalData		*ld;
	struct SocketList		*psl;

	ld  = (struct LocalData *)TlsGetValue(TlsIndex);
	psl = SocketListGetPos(s);
	
x_OutputDebugString("send ");

	if (psl && psl->STelChk.bSSLActive && !psl->STelChk.bInSSLCall){
x_OutputDebugString("S ");

		psl->STelChk.bInSSLCall = TRUE;
		ret = SSL_write(psl->STelChk.con, buf, len);
		psl->STelChk.bInSSLCall = FALSE;

	} else {
x_OutputDebugString("O ");

		ret = gft.send(s, buf, len, flags);

	}	

	return ret;
}


///////////////////////////////////////////////////////////////
//
//	socket
//
SOCKET PASCAL FAR EXPORT
socket (int af, int type, int protocol)
{
	SOCKET s;

x_OutputDebugString("socket ");
	
	s = gft.socket(af, type, protocol);

	SocketListAddTail(s);

	return s;
}


///////////////////////////////////////////////////////////////
//
//	WSAAsyncSelect
//
int PASCAL FAR EXPORT
WSAAsyncSelect(SOCKET s, HWND hWnd, u_int wMsg, long lEvent)
{
	int		ret;
	struct LocalData		*ld;
	ld = (struct LocalData *)TlsGetValue(TlsIndex);

x_OutputDebugString("WSAAsyncSelect ");
	
	ret = gft.WSAAsyncSelect(s, hWnd, wMsg, lEvent);

	ld->sAsyncSocket = s;
	ld->hAsyncWnd    = hWnd;
	ld->uAsyncMsg    = wMsg;
	ld->lAsyncEvent  = lEvent;

	return ret;
}


///////////////////////////////////////////////////////////////
//
//	WSAStartup
//
int PASCAL FAR EXPORT
WSAStartup(WORD wVersionRequired, LPWSADATA lpWSAData)
{
	int ret;

x_OutputDebugString("WSAStartup ");

	if (gft.hWinsock == NULL){
		if (LoadWinsock() == FALSE){
			return WSASYSNOTREADY;
		}
	}

	ret = gft.WSAStartup(wVersionRequired, lpWSAData);

	return ret;
}


///////////////////////////////////////////////////////////////
//
//	InitLocalData
//
int InitLocalData()
{
	struct LocalData		*ld;
	ld = (struct LocalData *)TlsGetValue(TlsIndex);

	ld->sAsyncSocket = 0;
	ld->hAsyncWnd    = NULL;
	ld->uAsyncMsg    = 0;
	ld->lAsyncEvent  = 0;

	return TRUE;
}


/////////////////////////////////////////////////////////////////////
//
//	DllMain
//
BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	struct LocalData		*ld;

	switch(ul_reason_for_call){
	case DLL_PROCESS_ATTACH:
		ghModule = hModule;

		TlsIndex = TlsAlloc();

		gft.hWinsock = NULL;

		ld = (struct LocalData *)LocalAlloc(LPTR, sizeof(struct LocalData));
		TlsSetValue(TlsIndex, (LPVOID)ld);
		ld->hWinsock = gft.hWinsock;

		InitLocalData();
		break;

	case DLL_THREAD_ATTACH:
		ld = (struct LocalData *)LocalAlloc(LPTR, sizeof(struct LocalData));
		TlsSetValue(TlsIndex, (LPVOID)ld);

		ld->hWinsock = LoadLibrary(ORIGINAL_WSOCK32);

		InitLocalData();
		break;

	case DLL_THREAD_DETACH:
		ld = (struct LocalData *)TlsGetValue(TlsIndex);

		FreeLibrary(ld->hWinsock);

		LocalFree(ld);
		break;

	case DLL_PROCESS_DETACH:
		ld = (struct LocalData *)TlsGetValue(TlsIndex);
		LocalFree(ld);

		FreeWinsock();

		TlsFree(TlsIndex);
		break;
	}
	return TRUE;
}


/////////////////////////////////////////////////////////////////////////
//
//	LoadWinsock
//
int LoadWinsock()
{
	gft.hWinsock = LoadLibrary(ORIGINAL_WSOCK32);

	if (gft.hWinsock == NULL){
		char szWSPath[MAX_PATH];

		if (!GetSystemDirectory(szWSPath, MAX_PATH))
			return FALSE;
		
		strcat(szWSPath, "\\wsock32.dll");

		gft.hWinsock = LoadLibrary(szWSPath);

	}

	if (gft.hWinsock == NULL)
		return FALSE;

	gft.accept                = (void *)GetProcAddress(gft.hWinsock, "accept");
	gft.bind                  = (void *)GetProcAddress(gft.hWinsock, "bind");
	gft.closesocket           = (void *)GetProcAddress(gft.hWinsock, "closesocket");
	gft.connect               = (void *)GetProcAddress(gft.hWinsock, "connect");
	gft.getpeername           = (void *)GetProcAddress(gft.hWinsock, "getpeername");
	gft.getsockname           = (void *)GetProcAddress(gft.hWinsock, "getsockname");
	gft.getsockopt            = (void *)GetProcAddress(gft.hWinsock, "getsockopt");
	gft.htonl                 = (void *)GetProcAddress(gft.hWinsock, "htonl");
	gft.htons                 = (void *)GetProcAddress(gft.hWinsock, "htons");
	gft.inet_addr             = (void *)GetProcAddress(gft.hWinsock, "inet_addr");
	gft.inet_ntoa             = (void *)GetProcAddress(gft.hWinsock, "inet_ntoa");
	gft.ioctlsocket           = (void *)GetProcAddress(gft.hWinsock, "ioctlsocket");
	gft.listen                = (void *)GetProcAddress(gft.hWinsock, "listen");
	gft.ntohl                 = (void *)GetProcAddress(gft.hWinsock, "ntohl");
	gft.ntohs                 = (void *)GetProcAddress(gft.hWinsock, "ntohs");
	gft.recv                  = (void *)GetProcAddress(gft.hWinsock, "recv");
	gft.recvfrom              = (void *)GetProcAddress(gft.hWinsock, "recvfrom");
	gft.select                = (void *)GetProcAddress(gft.hWinsock, "select");
	gft.send                  = (void *)GetProcAddress(gft.hWinsock, "send");
	gft.sendto                = (void *)GetProcAddress(gft.hWinsock, "sendto");
	gft.setsockopt            = (void *)GetProcAddress(gft.hWinsock, "setsockopt");
	gft.socket                = (void *)GetProcAddress(gft.hWinsock, "socket");
	gft.shutdown              = (void *)GetProcAddress(gft.hWinsock, "shutdown");

	gft.gethostbyaddr         = (void *)GetProcAddress(gft.hWinsock, "gethostbyaddr");
	gft.gethostbyname         = (void *)GetProcAddress(gft.hWinsock, "gethostbyname");
	gft.getprotobyname        = (void *)GetProcAddress(gft.hWinsock, "getprotobyname");
	gft.getprotobynumber      = (void *)GetProcAddress(gft.hWinsock, "getprotobynumber");
	gft.getservbyname         = (void *)GetProcAddress(gft.hWinsock, "getservbyname");
	gft.getservbyport         = (void *)GetProcAddress(gft.hWinsock, "getservbyport");
	gft.gethostname           = (void *)GetProcAddress(gft.hWinsock, "gethostname");

	gft.WSAAsyncSelect        = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncSelect");
	gft.WSAAsyncGetHostByAddr = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetHostByAddr");
	gft.WSAAsyncGetHostByName = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetHostByName");
	gft.WSAAsyncGetProtoByNumber = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetProtoByNumber");
	gft.WSAAsyncGetProtoByName = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetProtoByName");
	gft.WSAAsyncGetServByPort = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetServByPort");
	gft.WSAAsyncGetServByName = (void *)GetProcAddress(gft.hWinsock, "WSAAsyncGetServByName");
	gft.WSACancelAsyncRequest = (void *)GetProcAddress(gft.hWinsock, "WSACancelAsyncRequest");
	gft.WSASetBlockingHook    = (void *)GetProcAddress(gft.hWinsock, "WSASetBlockingHook");
	gft.WSAUnhookBlockingHook = (void *)GetProcAddress(gft.hWinsock, "WSAUnhookBlockingHook");
	gft.WSAGetLastError       = (void *)GetProcAddress(gft.hWinsock, "WSAGetLastError");
	gft.WSASetLastError       = (void *)GetProcAddress(gft.hWinsock, "WSASetLastError");
	gft.WSACancelBlockingCall = (void *)GetProcAddress(gft.hWinsock, "WSACancelBlockingCall");
	gft.WSAIsBlocking         = (void *)GetProcAddress(gft.hWinsock, "WSAIsBlocking");
	gft.WSAStartup            = (void *)GetProcAddress(gft.hWinsock, "WSAStartup");
	gft.WSACleanup            = (void *)GetProcAddress(gft.hWinsock, "WSACleanup");

	gft.__WSAFDIsSet          = (void *)GetProcAddress(gft.hWinsock, "__WSAFDIsSet");

	gft.EnumProtocolsA		  = (void *)GetProcAddress(gft.hWinsock, "EnumProtocolsA");
	gft.EnumProtocolsW		  = (void *)GetProcAddress(gft.hWinsock, "EnumProtocolsW");
	gft.GetAddressByNameA	  = (void *)GetProcAddress(gft.hWinsock, "GetAddressByNameA");
	gft.GetAddressByNameW	  = (void *)GetProcAddress(gft.hWinsock, "GetAddressByNameW");
	gft.GetNameByTypeA		  = (void *)GetProcAddress(gft.hWinsock, "GetNameByTypeA");
	gft.GetNameByTypeW		  = (void *)GetProcAddress(gft.hWinsock, "GetNameByTypeW");
	gft.GetServiceA			  = (void *)GetProcAddress(gft.hWinsock, "GetServiceA");
	gft.GetServiceW			  = (void *)GetProcAddress(gft.hWinsock, "GetServiceW");
	gft.GetTypeByNameA		  = (void *)GetProcAddress(gft.hWinsock, "GetTypeByNameA");
	gft.GetTypeByNameW		  = (void *)GetProcAddress(gft.hWinsock, "GetTypeByNameW");
	gft.SetServiceA			  = (void *)GetProcAddress(gft.hWinsock, "SetServiceA");
	gft.SetServiceW			  = (void *)GetProcAddress(gft.hWinsock, "SetServiceW");
	gft.TransmitFile		  = (void *)GetProcAddress(gft.hWinsock, "TransmitFile");
	gft.WSARecvEx			  = (void *)GetProcAddress(gft.hWinsock, "WSARecvEx");

	gft.WsControl             = (void *)GetProcAddress(gft.hWinsock, "WsControl");
	gft.closesockinfo         = (void *)GetProcAddress(gft.hWinsock, "closesockinfo");
	gft.Arecv                 = (void *)GetProcAddress(gft.hWinsock, "Arecv");
	gft.Asend                 = (void *)GetProcAddress(gft.hWinsock, "Asend");
	gft.WSHEnumProtocols      = (void *)GetProcAddress(gft.hWinsock, "WSHEnumProtocols");
	gft.inet_network          = (void *)GetProcAddress(gft.hWinsock, "inet_network");
	gft.getnetbyname          = (void *)GetProcAddress(gft.hWinsock, "getnetbyname");
	gft.rcmd                  = (void *)GetProcAddress(gft.hWinsock, "rcmd");
	gft.rexec                 = (void *)GetProcAddress(gft.hWinsock, "rexec");
	gft.rresvport             = (void *)GetProcAddress(gft.hWinsock, "rresvport");
	gft.sethostname           = (void *)GetProcAddress(gft.hWinsock, "sethostname");
	gft.dn_expand             = (void *)GetProcAddress(gft.hWinsock, "dn_expand");
	gft.s_perror              = (void *)GetProcAddress(gft.hWinsock, "s_perror");
	gft.NPLoadNameSpaces      = (void *)GetProcAddress(gft.hWinsock, "NPLoadNameSpaces");

x_OutputDebugString("END: LoadWinsock\n");

	return TRUE;
}


///////////////////////////////////////////////////////////////
//
//	FreeWinsock
//
int FreeWinsock()
{
	FreeLibrary(gft.hWinsock);

x_OutputDebugString("FreeWinsock\n");

	return TRUE;
}
